package net.vidageek.mirror;

import java.lang.reflect.AccessibleObject;

import net.vidageek.mirror.config.MirrorProviderBuilder;
import net.vidageek.mirror.exception.MirrorException;

/**
 * This is the basic class to use Mirror. All Reflection features can be
 * accessed using this class.
 * 
 * It will load configuration file [mirror.properties]. Full Mirror
 * configuration is found at
 * http://projetos.vidageek.net/mirror/extension/configuration/
 * 
 * @author jonasabreu
 */
public final class Mirror {

	private static final String MIRROR_CFG = "/mirror.properties";

	private static final MirrorInfrastructure mirror;

	static {
		mirror = new MirrorInfrastructure(new MirrorProviderBuilder(Mirror.class.getResource(MIRROR_CFG))
				.createProvider());
	}

	private Mirror() {
	}

	/**
	 * This method will reflect a Class object that is represented by className.
	 * 
	 * @param className
	 *            Full qualified name of the class you want to reflect.
	 * @return A Class object represented by className.
	 * @throws MirrorException
	 *             if no class on current ClassLoader has class represented by
	 *             className.
	 * @throws IllegalArgumentException
	 *             if className is null or empty.
	 */
	public static Class<?> reflectClass(final String className) {
		return mirror.reflectClass(className);
	}

	/**
	 * Method to access reflection on clazz.
	 * 
	 * @param clazz
	 *            Class object to be used.
	 * @return A object that allows you to access reflection on class clazz.
	 * @throws IllegalArgumentException
	 *             if clazz is null.
	 */
	public static <T> ClassController<T> on(final Class<T> clazz) {
		return mirror.on(clazz);
	}

	/**
	 * Method to access reflection on object.
	 * 
	 * @param object
	 *            Object to be used.
	 * @return A object that allows you to access reflection on object.
	 * @throws IllegalArgumentException
	 *             if object is null.
	 */
	public static ObjectController on(final Object object) {
		return mirror.on(object);
	}

	/**
	 * Convenience method for {@link Mirror#on(Class)}.
	 * 
	 * @see Mirror#on(Class)
	 * @see Mirror#reflectClass(String)
	 */
	public static ClassController<?> on(final String className) {
		return mirror.on(className);
	}

	/**
	 * Method to access reflection on any AccessibleObject
	 * 
	 * @see AccessibleObject
	 * @param object
	 *            AccessibleObject to be used.
	 * @return A object that allows you to access reflection on an
	 *         AccessibleObject.
	 */
	public static MemberController on(final AccessibleObject object) {
		return mirror.on(object);
	}

}
