package net.vidageek.mirror.reflect;

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;
import java.util.Arrays;

import net.vidageek.mirror.provider.AnnotatedElementReflectionProvider;
import net.vidageek.mirror.provider.ReflectionProvider;

/**
 * This class is responsible for reflecting annotations on a method.
 * 
 * @author dnfeitosa
 */
public final class MethodAnnotationHandler<T extends Annotation> {

	private final Class<?> clazz;

	private final String methodName;

	private final Class<T> annotation;

	private final ReflectionProvider provider;

	public MethodAnnotationHandler(final ReflectionProvider provider, final Class<?> clazz, final String methodName,
			final Class<T> annotation) {
		if (clazz == null) {
			throw new IllegalArgumentException("Argument clazz cannot be null.");
		}
		if (methodName == null || methodName.trim().length() == 0) {
			throw new IllegalArgumentException("Argument fieldName cannot be null or empty.");
		}
		if (annotation == null) {
			throw new IllegalArgumentException("Argument annotation cannot be null.");
		}
		this.provider = provider;
		this.clazz = clazz;
		this.methodName = methodName.trim();
		this.annotation = annotation;
	}

	/**
	 * Use this method to reflect a single annotation at the method that matches
	 * classes as its arguments.
	 * 
	 * @param classes
	 *            arguments of the method from where to search for annotations.
	 * @return The annotation or null if it was not found.
	 * @see AnnotatedElementReflectionProvider#getAnnotation(Class)
	 */
	public T withArgs(final Class<?>... classes) {
		Method method = new MethodReflector(provider, methodName, clazz).withArgs(classes);
		if (method == null) {
			throw new IllegalArgumentException("could not find method matching argument list " + Arrays.asList(classes));
		}
		return provider.getAnnotatedElementReflectionProvider(method).getAnnotation(annotation);
	}

	/**
	 * This is a convenience method for
	 * {@link MethodAnnotationHandler#withArgs(Class...)}
	 * 
	 * @see MethodAnnotationHandler#withArgs(Class...)
	 */
	public T withoutArgs() {
		return withArgs(new Class<?>[0]);
	}
}
