/**
 * 
 */
package net.vidageek.mirror.reflect;

import java.lang.reflect.Field;

import net.vidageek.mirror.provider.ReflectionProvider;

/**
 * This part of the DSL is used to choose reflection element is going to be
 * reflected.
 * 
 * @author jonasabreu
 */
public final class ReflectionHandler<T> {

	private final Class<T> clazz;

	private final ReflectionProvider provider;

	public ReflectionHandler(final ReflectionProvider provider, final Class<T> clazz) {
		if (clazz == null) {
			throw new IllegalArgumentException("clazz cannot be null");
		}
		this.provider = provider;
		this.clazz = clazz;
	}

	/**
	 * Use this method to reflect a field by its name.
	 * 
	 * @param fieldName
	 *            Name of the field to be reflected.
	 * @return A field identified by fieldName or null if none is found.
	 * @throws IllegalArgumentException
	 *             if fieldName is null or empty
	 * @see FieldReflector#onClass(Class)
	 */
	public Field field(final String fieldName) {
		if (fieldName == null || fieldName.trim().length() == 0) {
			throw new IllegalArgumentException("fieldName cannot be null or empty.");
		}
		return new FieldReflector(provider, fieldName).onClass(clazz);
	}

	/**
	 * Use this method to choose the name of the method to reflect.
	 * 
	 * @param methodName
	 *            name of the method to be reflected
	 * @return An object responsible for reflecting the method.
	 * @throws IllegalArgumentException
	 *             if methodName is null or empty
	 * @see MethodReflector
	 */
	public MethodReflector method(final String methodName) {
		if (methodName == null || methodName.trim().length() == 0) {
			throw new IllegalArgumentException("methodName cannot be null or empty.");
		}
		return new MethodReflector(provider, methodName, clazz);
	}

	/**
	 * Use this method to reflect a constructor
	 * 
	 * @return An object responsible for reflecting constructors.
	 * @see ConstructorReflector
	 */
	public ConstructorReflector<T> constructor() {
		return new ConstructorReflector<T>(provider, clazz);
	}

	/**
	 * Use this method to reflect annotations.
	 * 
	 * @param annotation
	 *            Type of the annotation to be reflected.
	 * @return An object responsible for reflecting annotations.
	 * @see AnnotationHandler
	 */
	@SuppressWarnings("unchecked")
	public <A> AnnotationHandler<? extends A> annotation(final Class<A> annotation) {
		return new AnnotationHandler(provider, clazz, annotation);
	}

}
